@@ -21,7 +21,6 @@
 
 #include <config.h>
 #include <errno.h>
-#include <math.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -413,8 +412,7 @@ pango_scan_string (const char **pos, GString *out)
  * @pos: in/out string position
  * @out: an int into which to write the result
  * 
- * Scans an integer. An integer consists
- * of up to 31 decimal digits. 
+ * Scans an integer. 
  * Leading white space is skipped.
  * 
  * Return value: %FALSE if a parse error occured.
@@ -423,29 +421,24 @@ gboolean
 pango_scan_int (const char **pos, int *out)
 {
   unsigned int i = 0;
-  char buf[32];
-  const char *p = *pos;
+  char *end;
+  long temp;
 
-  while (g_ascii_isspace (*p))
-    p++;
-  
-  if (*p < '0' || *p > '9')
-    return FALSE;
-
-  while ((*p >= '0') && (*p <= '9') && i < sizeof(buf))
+  errno = 0;
+  temp = strtol (*pos, &end, 10);
+  if (errno == ERANGE)
     {
-      buf[i] = *p;
-      i++;
-      p++;
+      errno = 0;
+      return FALSE;
+    }
+	
+  *out = (int)temp;
+  if ((long)(*out) != temp)
+    {
+      return FALSE;
     }
 
-  if (i == sizeof(buf))
-    return FALSE;
-  else
-    buf[i] = '\0';
-
-  *out = atoi (buf);
-  *pos = p;
+  *pos = end;
 
   return TRUE;
 }
@@ -929,235 +922,11 @@ pango_parse_stretch (const char   *str,
   return FALSE;
 }
 
-GType
-pango_matrix_get_type (void)
-{
-  static GType our_type = 0;
-  
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static (I_("PangoMatrix"),
-					     (GBoxedCopyFunc) pango_matrix_copy,
-					     (GBoxedFreeFunc) pango_matrix_free);
-
-  return our_type;
-}
-
-/**
- * pango_matrix_copy:
- * @matrix: a #PangoMatrix, can be %NULL
- * 
- * Copies a #PangoMatrix.
- * 
- * Return value: the newly allocated #PangoMatrix, which should
- *               be freed with pango_matrix_free(), or %NULL if
- *               @matrix was %NULL.
- *
- * Since: 1.6
- **/
-PangoMatrix *
-pango_matrix_copy (const PangoMatrix *matrix)
-{
-  PangoMatrix *new_matrix;
-
-  if (matrix)
-    {
-      new_matrix = g_slice_new (PangoMatrix);
-      *new_matrix = *matrix;
-    }
-  else
-    new_matrix = NULL;
-
-  return new_matrix;
-}
-
-/**
- * pango_matrix_free:
- * @matrix: a #PangoMatrix, or %NULL
- * 
- * Free a #PangoMatrix created with pango_matrix_copy().
- * Does nothing if @matrix is %NULL.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_free (PangoMatrix *matrix)
-{
-  if (matrix)
-    g_slice_free (PangoMatrix, matrix);
-}
-
-/**
- * pango_matrix_translate:
- * @matrix: a #PangoMatrix
- * @tx: amount to translate in the X direction
- * @ty: amount to translate in the Y direction
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first translating by (@tx, @ty)
- * then applying the original transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_translate (PangoMatrix *matrix,
-			double       tx,
-			double       ty)
-{
-  g_return_if_fail (matrix != NULL);
-
-  matrix->x0  = matrix->xx * tx + matrix->xy * ty + matrix->x0;
-  matrix->y0  = matrix->yx * tx + matrix->yy * ty + matrix->y0;
-}
-
-/**
- * pango_matrix_scale:
- * @matrix: a #PangoMatrix
- * @scale_x: amount to scale by in X direction
- * @scale_y: amount to scale by in Y direction
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first scaling by @sx in the X direction
- * and @sy in the Y direction then applying the original
- * transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_scale (PangoMatrix *matrix,
-		    double       scale_x,
-		    double       scale_y)
-{
-  g_return_if_fail (matrix != NULL);
-
-  matrix->xx *= scale_x;
-  matrix->xy *= scale_y;
-  matrix->yx *= scale_x;
-  matrix->yy *= scale_y;
-}
-
-/**
- * pango_matrix_rotate:
- * @matrix: a #PangoMatrix
- * @degrees: degrees to rotate counter-clockwise
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first rotating by @degrees degrees
- * counter-clokwise then applying the original transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_rotate (PangoMatrix *matrix,
-		     double       degrees)
-{
-  PangoMatrix tmp;
-  gdouble r, s, c;
-
-  g_return_if_fail (matrix != NULL);
-
-  r = degrees * (G_PI / 180.);
-  s = sin (r);
-  c = cos (r);
-
-  tmp.xx = c;
-  tmp.xy = s;
-  tmp.yx = -s;
-  tmp.yy = c;
-  tmp.x0 = 0;
-  tmp.y0 = 0;
-
-  pango_matrix_concat (matrix, &tmp);
-}
-
-/**
- * pango_matrix_concat:
- * @matrix: a #PangoMatrix
- * @new_matrix: a #PangoMatrix
- * 
- * Changes the transformation represented by @matrix to be the
- * transformation given by first applying transformation
- * given by @new_matrix then applying the original transformation.
- *
- * Since: 1.6
- **/
-void
-pango_matrix_concat (PangoMatrix       *matrix,
-		     const PangoMatrix *new_matrix)
-{
-  PangoMatrix tmp;
-  
-  g_return_if_fail (matrix != NULL);
-
-  tmp = *matrix;
-
-  matrix->xx = tmp.xx * new_matrix->xx + tmp.xy * new_matrix->yx;
-  matrix->xy = tmp.xx * new_matrix->xy + tmp.xy * new_matrix->yy;
-  matrix->yx = tmp.yx * new_matrix->xx + tmp.yy * new_matrix->yx;
-  matrix->yy = tmp.yx * new_matrix->xy + tmp.yy * new_matrix->yy;
-  matrix->x0  = tmp.xx * new_matrix->x0 + tmp.xy * new_matrix->y0 + tmp.x0;
-  matrix->y0  = tmp.yx * new_matrix->y0 + tmp.yy * new_matrix->y0 + tmp.y0;
-}
-
-/**
- * pango_matrix_get_font_scale_factor:
- * @matrix: a #PangoMatrix, may be %NULL
- * 
- * Returns the scale factor of a matrix on the height of the font.
- * That is, the scale factor in the direction perpendicular to the
- * vector that the X coordinate is mapped to.
- *
- * Return value: the scale factor of @matrix on the height of the font,
- * or 1.0 if @matrix is %NULL.
- *
- * Since: 1.12
- **/
-double
-pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
-{
-/*
- * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
- *
- * Copyright 2005, Keith Packard
- */
-  double det;
-  
-  if (!matrix)
-    return 1.0;
-  
-  det = matrix->xx * matrix->yy - matrix->yx * matrix->xy;
-
-  if (det == 0)
-    {
-      return 0.0;
-    }
-  else
-    {
-      double x = matrix->xx;
-      double y = matrix->yx;
-      double major, minor;
-
-      major = sqrt (x*x + y*y);
-      
-      /*
-       * ignore mirroring
-       */
-      if (det < 0)
-	det = - det;
-      
-      if (major)
-	minor = det / major;
-      else 
-	minor = 0.0;
-
-      return minor;
-    }
-}
-
 static const char canon_map[256] = {
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,  '-',  0,   0, 
-   0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
+  '0', '1', '2', '3', '4', '5', '6', '7',  '8', '9',  0,   0,   0,   0,   0,   0, 
    0,  'a', 'b', 'c', 'd', 'e', 'f', 'g',  'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
   'p', 'q', 'r', 's', 't', 'u', 'v', 'w',  'x', 'y', 'z',  0,   0,   0,   0,  '-',
    0,  'a', 'b', 'c', 'd', 'e', 'f', 'g',  'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
@@ -1171,15 +940,12 @@ lang_equal (gconstpointer v1,
   const guchar *p1 = v1;
   const guchar *p2 = v2;
 
-  while (*p2)
+  while (canon_map[*p1] && canon_map[*p1] == canon_map[*p2])
     {
-      guchar value = canon_map[*p2];
-      if (value && value != *p1++)
-	return FALSE;
-      p2++;
+      p1++, p2++;
     }
 
-  return (*p1 == '\0');
+  return (canon_map[*p1] == canon_map[*p2]);
 }
 
 static guint
@@ -1187,11 +953,9 @@ lang_hash (gconstpointer key)
 {
   const guchar *p = key;
   guint h = 0;
-  while (*p)
+  while (canon_map[*p])
     {
-      guchar value = canon_map[*p];
-      if (value)
-	h = (h << 5) - h + value;
+      h = (h << 5) - h + canon_map[*p];
       p++;
     }
 
@@ -1246,24 +1010,21 @@ pango_language_from_string (const char *language)
   int len;
   char *p;
 
-  if (!hash)
+  if (G_UNLIKELY (!hash))
     hash = g_hash_table_new (lang_hash, lang_equal);
-
-  result = g_hash_table_lookup (hash, language);
-  if (result)
-    return (PangoLanguage *)result;
+  else
+    {
+      result = g_hash_table_lookup (hash, language);
+      if (result)
+	return (PangoLanguage *)result;
+    }
 
   len = strlen (language);
   result = g_malloc (len + 1);
 
   p = result;
-  while (*language)
-    {
-      char value = canon_map[*(guchar *)language++];
-      if (value)
-	*(p++) = value;
-    }
-  *p++ = '\0';
+  while ((*(p++) = canon_map[*(guchar *)language++]))
+    ;
 
   g_hash_table_insert (hash, result, result);
 
@@ -1274,15 +1035,16 @@ pango_language_from_string (const char *language)
  * pango_language_matches:
  * @language: a language tag (see pango_language_from_string()),
  *            %NULL is allowed and matches nothing but '*'
- * @range_list: a list of language ranges, separated by ';' characters.
- *   each element must either be '*', or a RFC 3066 language range
+ * @range_list: a list of language ranges, separated by ';', ':',
+ *   ',', or space characters.
+ *   Each element must either be '*', or a RFC 3066 language range
  *   canonicalized as by pango_language_from_string()
  * 
  * Checks if a language tag matches one of the elements in a list of
  * language ranges. A language tag is considered to match a range
  * in the list if the range is '*', the range is exactly the tag,
- * or the range is a prefix of the tag, and the character after the
- * tag is '-'.
+ * or the range is a prefix of the tag, and the character after it
+ * in the tag is '-'.
  *
  * Return value: %TRUE if a match was found.
  **/
@@ -1296,7 +1058,7 @@ pango_language_matches (PangoLanguage *language,
 
   while (!done)
     {
-      const char *end = strchr (p, ';');
+      const char *end = strpbrk (p, ";:, \t");
       if (!end)
 	{
 	  end = p + strlen (p);
@@ -1762,7 +1524,7 @@ pango_find_base_dir (const gchar *text,
   PangoDirection dir = PANGO_DIRECTION_NEUTRAL;
   const gchar *p;
 
-  g_return_val_if_fail (text != NULL, PANGO_DIRECTION_NEUTRAL);
+  g_return_val_if_fail (text != NULL || length == 0, PANGO_DIRECTION_NEUTRAL);
 
   p = text;
   while ((length < 0 || p < text + length) && *p)
@@ -1795,25 +1557,39 @@ gboolean
 pango_is_zero_width (gunichar ch)
 {
 /* Zero Width characters:
+ *
+ *  00AD  SOFT HYPHEN
+ *  034F  COMBINING GRAPHEME JOINER
  *
  *  200B  ZERO WIDTH SPACE
  *  200C  ZERO WIDTH NON-JOINER
  *  200D  ZERO WIDTH JOINER
  *  200E  LEFT-TO-RIGHT MARK
  *  200F  RIGHT-TO-LEFT MARK
+ *
  *  2028  LINE SEPARATOR
+ *
  *  202A  LEFT-TO-RIGHT EMBEDDING
  *  202B  RIGHT-TO-LEFT EMBEDDING
  *  202C  POP DIRECTIONAL FORMATTING
  *  202D  LEFT-TO-RIGHT OVERRIDE
  *  202E  RIGHT-TO-LEFT OVERRIDE
+ *
+ *  2060  WORD JOINER
+ *  2061  FUNCTION APPLICATION
+ *  2062  INVISIBLE TIMES
+ *  2063  INVISIBLE SEPARATOR
+ *
  *  FEFF  ZERO WIDTH NO-BREAK SPACE
  */
-  return ((ch & ~(gunichar)0x003F) == 0x2000 && (
+  return ((ch & ~(gunichar)0x007F) == 0x2000 && (
 		(ch >= 0x200B && ch <= 0x200F) ||
-		ch == 0x2028 ||
-		(ch >= 0x202A && ch <= 0x202E)
-	 )) || ch == 0xFEFF;
+		(ch >= 0x202A && ch <= 0x202E) ||
+		(ch >= 0x2060 && ch <= 0x2063) ||
+		(ch == 0x2028)
+	 )) || G_UNLIKELY (ch == 0x00AD
+			|| ch == 0x034F
+			|| ch == 0xFEFF);
 }
 
 /**
